@@ -4,7 +4,12 @@ from typing import Any, TypeVar
 import pytest
 import voluptuous as vol
 
-from voluptuous_openapi import UNSUPPORTED, convert, convert_to_voluptuous
+from voluptuous_openapi import (
+    UNSUPPORTED,
+    convert,
+    convert_to_voluptuous,
+    OpenApiVersion,
+)
 
 
 def test_int_schema():
@@ -226,6 +231,15 @@ def test_maybe():
     } == convert(vol.Schema(vol.Maybe(str)))
 
 
+def test_maybe_v3_1():
+    assert {
+        "anyOf": [
+            {"type": "null"},
+            {"type": "string"},
+        ],
+    } == convert(vol.Schema(vol.Maybe(str)), openapi_version=OpenApiVersion.V3_1)
+
+
 def test_custom_serializer():
     def custem_serializer(schema):
         if schema is str:
@@ -257,6 +271,12 @@ def test_constant():
         "nullable": True,
         "description": "Must be null",
     } == convert(vol.Schema(type(None)))
+    assert {
+        "type": "null",
+    } == convert(vol.Schema(None), openapi_version=OpenApiVersion.V3_1)
+    assert {
+        "type": "null",
+    } == convert(vol.Schema(type(None)), openapi_version=OpenApiVersion.V3_1)
 
 
 def test_enum():
@@ -327,6 +347,11 @@ def test_any_of():
         "anyOf": [{"type": "number"}, {"type": "integer"}],
         "nullable": True,
     } == convert(vol.Any(vol.Maybe(float), vol.Maybe(int)))
+    assert {
+        "anyOf": [{"type": "null"}, {"type": "number"}, {"type": "integer"}],
+    } == convert(
+        vol.Any(vol.Maybe(float), vol.Maybe(int)), openapi_version=OpenApiVersion.V3_1
+    )
 
 
 def test_all_of():
@@ -421,6 +446,9 @@ def test_function():
     assert {"type": "number", "nullable": True} == convert(
         vol.Schema(validator_nullable)
     )
+    assert {"anyOf": [{"type": "number"}, {"type": "null"}]} == convert(
+        vol.Schema(validator_nullable), openapi_version=OpenApiVersion.V3_1
+    )
 
     def validator_union(data: float | int):
         return data
