@@ -2,7 +2,7 @@
 
 from collections.abc import Callable, Mapping, Sequence
 from inspect import signature
-from enum import Enum
+from enum import Enum, StrEnum
 import re
 from typing import Any, TypeVar, Union, get_args, get_origin, get_type_hints
 from types import NoneType, UnionType
@@ -22,7 +22,6 @@ UNSUPPORTED = object()
 
 # These are not supported when converting from OpenAPI to voluptuous
 OPENAPI_UNSUPPORTED_KEYWORDS = {
-    "anyOf",
     "allOf",
     "multipleOf",
     "minItems",
@@ -31,10 +30,31 @@ OPENAPI_UNSUPPORTED_KEYWORDS = {
 }
 
 
-def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
+class OpenApiVersion(StrEnum):
+    """The OpenAPI version.
+
+    This is used to change the behavior when converting schemas to OpenAPI.
+    """
+
+    V3 = "3.0"
+    V3_1 = "3.1.0"
+
+
+def convert(
+    schema: Any,
+    *,
+    custom_serializer: Callable | None = None,
+    openapi_version: OpenApiVersion = OpenApiVersion.V3,
+) -> dict:
     """Convert a voluptuous schema to a OpenAPI Schema object."""
     # pylint: disable=too-many-return-statements,too-many-branches
 
+    def convert_with_args(schema: Any) -> dict:
+        """Convert schema for recusing and propagating arguments."""
+        return convert(
+            schema, custom_serializer=custom_serializer, openapi_version=openapi_version
+        )
+
     def ensure_default(value: dict[str:Any]):
         """Make sure that type is set."""
         if all(x not in value for x in ("type", "anyOf", "oneOf", "allOf", "not")):
@@ -77,7 +97,7 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
             else:
                 pkey = key
 
-            pval = convert(value, custom_serializer=custom_serializer)
+            pval = convert_with_args(value)
             if description:
                 pval["description"] = key.description
 
@@ -122,7 +142,7 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
         fallback = False
         allOf = []
         for validator in schema.validators:
-            v = convert(validator, custom_serializer=custom_serializer)
+            v = convert_with_args(validator)
             if (
                 not v
                 or v in allOf
@@ -208,17 +228,19 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
 
     if isinstance(schema, vol.Any):
         schema = schema.validators
-        if None in schema or NoneType in schema:
+        # Infer the enum type based on the type of the first value, but default
+        # to a string as a fallback.
+        if (
+            None in schema or NoneType in schema
+        ) and openapi_version == OpenApiVersion.V3:
             schema = [val for val in schema if val is not None and val is not NoneType]
             nullable = True
         else:
             nullable = False
         if len(schema) == 1:
-            result = convert(schema[0], custom_serializer=custom_serializer)
+            result = convert_with_args(schema[0])
         else:
-            anyOf = [
-                convert(val, custom_serializer=custom_serializer) for val in schema
-            ]
+            anyOf = [convert_with_args(val) for val in schema]
 
             # Merge nested anyOf
             tmpAnyOf = []
@@ -302,6 +324,8 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
         return {"type": TYPES_MAP[type(schema)], "enum": [schema]}
 
     if schema is None:
+        if openapi_version == OpenApiVersion.V3_1:
+            return {"type": "null"}
         return {"type": "object", "nullable": True, "description": "Must be null"}
 
     if (
@@ -315,16 +339,11 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
         if len(schema) == 1:
             return {
                 "type": "array",
-                "items": ensure_default(
-                    convert(schema[0], custom_serializer=custom_serializer)
-                ),
+                "items": ensure_default(convert_with_args(schema[0])),
             }
         return {
             "type": "array",
-            "items": [
-                ensure_default(convert(s, custom_serializer=custom_serializer))
-                for s in schema.items()
-            ],
+            "items": [ensure_default(convert_with_args(s)) for s in schema.items()],
         }
 
     if schema in TYPES_MAP:
@@ -334,7 +353,7 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
         if get_args(schema)[1] is Any or isinstance(get_args(schema)[1], TypeVar):
             schema = dict
         else:
-            return convert({get_args(schema)[0]: get_args(schema)[1]})
+            return convert_with_args({get_args(schema)[0]: get_args(schema)[1]})
 
     if isinstance(schema, type):
         if schema is dict:
@@ -360,6 +379,8 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
                 return {"type": enum_type, "enum": enum_values, "nullable": True}
             return {"type": enum_type, "enum": enum_values}
         elif schema is NoneType:
+            if openapi_version == OpenApiVersion.V3_1:
+                return {"type": "null"}
             return {"type": "object", "nullable": True, "description": "Must be null"}
 
     if schema is object:
@@ -380,7 +401,7 @@ def convert(schema: Any, *, custom_serializer: Callable | None = None) -> dict:
             else:
                 return {}
 
-        return ensure_default(convert(schema, custom_serializer=custom_serializer))
+        return ensure_default(convert_with_args(schema))
 
     raise ValueError("Unable to convert schema: {}".format(schema))
 
@@ -398,11 +419,21 @@ def convert_to_voluptuous(schema: dict) -> vol.Schema:
     if (one_of := schema.get("oneOf")) is not None:
         if not isinstance(one_of, list):
             raise ValueError("Invalid schema, oneOf should be a list")
+        # This implements anyOf semantics sice it matches any of the subschemas,
+        # not just one of them.
         return vol.Any(*[convert_to_voluptuous(sub_schema) for sub_schema in one_of])
 
+    if (any_of := schema.get("anyOf")) is not None:
+        if not isinstance(any_of, list):
+            raise ValueError("Invalid schema, anyOf should be a list")
+        return vol.Any(*[convert_to_voluptuous(sub_schema) for sub_schema in any_of])
+
     if (schema_type := schema.get("type")) is None:
         raise ValueError("Invalid schema, missing type")
 
+    if schema["type"] == "null":
+        return vol.Schema(None)
+
     if (basic_type := TYPES_MAP_REV.get(schema_type)) is not None:
         if schema_type == "string":
             if (pattern := schema.get("pattern")) is not None:
